<?php
/**
 * Plugin Name: Tickera - Event Stats Shortcode
 * Plugin URI: http://tickera.com/
 * Description: Displays number of sold/left tickets for the upcoming / past events with [tc_custom_event_stats type="upcoming"] or [tc_custom_event_stats type="past"] shortcodes and using the event id with [tc_custom_event_stats event_id="123"]
 * Author: Tickera.com
 * Author URI: http://tickera.com/
 * Version: 1.0
 * TextDomain: tc_stats
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) )
    exit; // Exit if accessed directly

if ( ! class_exists( 'TC_Event_Stats_Shortcode' ) ) {

    class TC_Event_Stats_Shortcode {

        var $version = '1.0' ;
        var $name = 'tc_stats';

        function __construct() {
            add_shortcode( 'tc_custom_event_stats', array( &$this, 'tc_custom_event_stats' ) );
        }

        /**
         * Render Event Stats for Tickera Standalone
         *
         * @param $atts
         */
        function tc_event_stats( $atts ) {

            global $tc; // Required in order to use get_cart_currency_and_format method from core of Tickera

            extract( shortcode_atts( [
                'type' => 'upcoming', // Upcoming or past events
                'event_id' => '',
            ], $atts ) );

            if ( $event_id ) {

                $args = array (
                    'p' => $event_id,
                    'post_type' => 'tc_events',
                    'post_status' => 'publish'
                );

            } else {

                $args = array (
                    'post_type' => 'tc_events',
                    'meta_query' => array (
                        array (
                            'key' => 'event_date_time',
                            'value' => date( 'Y-m-d h:i' ),
                            'type' => 'DATETIME',
                            'compare' => ( 'past' == $type ) ? '<=' : '>='
                        ),
                        'orderby' => 'event_date_time',
                    ),
                    'order' => 'ASC',
                    'orderby' => 'meta_value',
                    'post_status' => 'publish',
                    'posts_per_page' => -1,
                );
            }

            $the_query = new WP_Query( $args );

            if ( $the_query->have_posts() ) {

                while ( $the_query->have_posts() ) {
                    $the_query->the_post();
                    $event = new TC_Event( get_the_ID() ); // Create event object
                    $event_location = $event->details->event_location;
                    $event_date = do_shortcode( '[tc_event_date event_id="' . get_the_ID() . '"]' ); // Shortcode from Tickera which shows full date of the event
                    ?>
                    <div class="tc-single-event">
                        <h3>
                            <a href="<?php esc_url( the_permalink() ); ?>"><?php echo esc_html( $event->details->post_title ); ?></a>
                            </br>
                            <span class="tc_event_date_title_front"><i class="fa fa-clock-o"></i><?php echo $event_date; ?> </span></br>
                            <?php if ( ! empty( $event_location ) ) { // Show only if the event location isn't empty ?>
                                <span class="tc_event_location_title_front"><i class="fa fa-map-marker"></i><?php echo $event_location; ?></span>
                            <?php } ?>
                        </h3>
                        <?php echo __('Event tickets sold: ', 'tc_stats') . do_shortcode( '[event_tickets_sold]' ); ?><br/>
                        <?php echo __('Event tickets left: ', 'tc_stats') . do_shortcode( '[event_tickets_left]' ); ?><br/><br/>
                        <?php echo __('Ticket types stats:', 'tc_stats')?><br/><br/>
                        <?php
                        $ticket_types = $event->get_event_ticket_types(); // Get all ticket types IDs associated with the event
                        if ( count( $ticket_types ) > 0 ) { ?>
                            <table>
                            <tr>
                                <th><?php _e('Ticket Type', 'tc_stats'); ?></th>
                                <th><?php _e('Count Left', 'tc_stats'); ?></th>
                                <th><?php _e('Count Sold', 'tc_stats'); ?></th>
                                <th><?php _e('Revenue', 'tc_stats'); ?></th>
                            </tr>
                            <?php foreach ( $ticket_types as $ticket_type_id ) {
                                $ticket_type = new TC_Ticket( $ticket_type_id ); // Create ticket type object so we can use its methods and properties
                                $count_left = $ticket_type->get_tickets_quantity_left();
                                $count_sold = $ticket_type->get_number_of_sold_tickets();
                                $price = $ticket_type->details->price_per_ticket;
                                ?>
                                <tr>
                                    <td><?php echo $ticket_type->details->post_title; ?></td>
                                    <td><?php echo $count_left == 9999 ? '∞' : $count_left; // Make sure to put a "∞" for qty 9999 because it means unlimited ?></td>
                                    <td><?php echo $count_sold; ?></td>
                                    <td><?php echo $tc->get_cart_currency_and_format( $count_sold * $price ); ?></td>
                                </tr>
                            <?php } ?>
                            </table><?php

                        } else {
                            echo __('Stats are not available since there are no ticket types associated with the event', 'tc_stats');
                        } ?>
                        <hr/>
                    </div>
                    <?php
                }

            } else { ?>
                <p><?php echo ( ! $event_id ) ? __('There are no ', 'tc_stats') . $type . __(' event available.', 'tc_stats') : __('There are no events available.', 'tc_stats'); ?></p><?php
            }

            // Restore original Post Data
            wp_reset_postdata();
        }

        /**
         * Render Event Stats with Bridge for Woocommerce
         *
         * @param $atts
         */
        function woo_event_stats( $atts ) {

            extract( shortcode_atts( [
                    'type' => 'upcoming', // Upcoming or past
                    'event_id' => '',
                ], $atts )
            );

            if ( $event_id ) {

                $args = array (
                    'p' => $event_id,
                    'post_type' => 'tc_events',
                    'post_status' => 'publish'
                );

            } else {

                $args = array (
                    'post_type' => 'tc_events',
                    'meta_query' => array (
                        array (
                            'key' => 'event_date_time',
                            'value' => date( 'Y-m-d h:i' ),
                            'type' => 'DATETIME',
                            'compare' => ( 'past' == $type ) ? '<=' : '>='
                        ),
                        'orderby' => 'event_date_time',
                    ),
                    'order' => 'ASC',
                    'orderby' => 'meta_value',
                    'post_status' => 'publish',
                    'posts_per_page' => -1
                );
            }

            $the_query = new WP_Query( $args );

            if ( $the_query->have_posts() ) {

                while ( $the_query->have_posts() ) {

                    $the_query->the_post();
                    $event = new TC_Event( get_the_ID() ); // Create event object
                    $event_location = $event->details->event_location;
                    $event_date = do_shortcode( '[tc_event_date event_id="' . get_the_ID() . '"]' );
                    $total_sold_ticket = self::wc_get_event_tickets_count_sold( get_the_ID() );
                    $total_left_ticket = self::wc_get_event_tickets_count_left( get_the_ID() );
                    ?>
                    <div class="tc-single-event">
                        <h3>
                            <a href="<?php esc_url( the_permalink() ); ?>"><?php echo esc_html( $event->details->post_title ); ?></a>
                            </br>
                            <span class="tc_event_date_title_front"><i class="fa fa-clock-o"></i><?php echo $event_date; ?> </span> </br>
                            <?php if ( ! empty( $event_location ) ) { // Show only if the event location isn't empty ?>
                                <span class="tc_event_location_title_front"><i class="fa fa-map-marker"></i><?php echo $event_location; ?></span>
                            <?php } ?>
                        </h3>
                        <?php echo __('Event tickets sold: ', 'tc_stats') . $total_sold_ticket ?><br/>
                        <?php echo __('Event tickets left: ', 'tc_stats') . $total_left_ticket ?><br/><br/>
                        <?php echo __('Ticket types stats:', 'tc_stats')?><br/><br/>
                        <?php
                        $ticket_types = $event->get_event_ticket_types();
                        if ( count( $ticket_types ) > 0 ) { ?>
                            <table>
                                <tr>
                                    <th><?php _e('Ticket Type', 'tc_stats'); ?></th>
                                    <th><?php _e('Count Left', 'tc_stats'); ?></th>
                                    <th><?php _e('Count Sold', 'tc_stats'); ?></th>
                                    <th><?php _e('Revenue', 'tc_stats'); ?></th>
                                </tr>
                                <?php foreach ( $ticket_types as $ticket_type_id ) {

                                    if ( in_array( get_post_type( $ticket_type_id ), [ 'product', 'product_variation' ] )
                                        && ! self::ticket_has_child( $ticket_type_id ) ) {

                                        $ticket_type = wc_get_product( $ticket_type_id );
                                        $count_sold = self::woo_get_total_sales( $ticket_type_id );
                                        $ticket_price = $ticket_type->get_price();
                                        $currency = get_woocommerce_currency_symbol();
                                        $count_left = $ticket_type->get_stock_quantity();

                                        if ( ! $ticket_type->managing_stock() && 'instock' == $ticket_type->get_stock_status() ) {
                                            $count_left = '∞';

                                        } elseif ( ! $ticket_type->managing_stock() && 'outofstock' == $ticket_type->get_stock_status() ) {
                                            $count_left = 0;
                                        }
                                        ?>
                                        <tr>
                                            <td><?php echo $ticket_type->get_name(); ?></td>
                                            <td><?php echo $count_left; ?></td>
                                            <td><?php echo $count_sold; ?></td>
                                            <td><?php echo $currency . ( $ticket_price * $count_sold ); ?></td>
                                        </tr>
                                        <?php
                                    }
                            } ?>
                            </table><?php

                        } else {
                            echo __('Stats are not available since there are no ticket types associated with this event', 'tc_stats)');
                        } ?>
                        <hr/>
                    </div><?php
                }

            } else { ?>
                <p><?php echo ( ! $event_id ) ? __('There are no ', 'tc_stats') . $type . __(' event available.', 'tc_stats') : __('There are no events available.', 'tc_stats'); ?></p><?php
            }

            // Restore original Post Data
            wp_reset_postdata();
        }

        /**
         * Get the total sales of a specific ticket_type/Product
         *
         * @param $ticket_type_id
         * @return int
         */
        function woo_get_total_sales( $ticket_type_id ) {

            global $wpdb;

            $sold_count = 0;

            $query = $wpdb->prepare( "SELECT COUNT(*) as cnt, p.post_parent FROM {$wpdb->posts} p, {$wpdb->postmeta} pm WHERE p.ID = pm.post_id AND pm.meta_key = 'ticket_type_id' AND pm.meta_value = %d GROUP BY p.post_parent", $ticket_type_id );
            $sold_records = $wpdb->get_results( $query );

            foreach ( $sold_records as $sold_record ) {

                $valid_order_status = [ 'wc-completed', 'wc-processing', 'wc-pending' ];
                $order_status = get_post_status( $sold_record->post_parent );

                if ( in_array( $order_status, $valid_order_status ) && 'shop_order' == get_post_type( $sold_record->post_parent ) ) {
                    $sold_count = $sold_count + $sold_record->cnt;
                }
            }

            return $sold_count;
        }

        /**
         * Get the total number of sold tickets per event basis
         *
         * @param $event_id
         * @return int
         */
        function wc_get_event_tickets_count_sold( $event_id ) {

            global $wpdb;

            $event = new TC_Event( $event_id );
            $ticket_types = $event->get_event_ticket_types();

            $sold_count = 0;
            if ( count($ticket_types) > 0 ) {

                $query = "SELECT COUNT(*) as cnt, p.post_parent FROM {$wpdb->posts} p, {$wpdb->postmeta} pm WHERE p.ID = pm.post_id AND pm.meta_key = 'ticket_type_id' AND pm.meta_value IN (" . implode( ',', $ticket_types ) . ") GROUP BY p.post_parent";
                $sold_records = $wpdb->get_results( $query );

                foreach ( $sold_records as $sold_record ) {

                    $valid_order_status = [ 'wc-completed', 'wc-processing', 'wc-pending' ];
                    $order_status = get_post_status( $sold_record->post_parent );

                    if ( in_array( $order_status, $valid_order_status ) && 'shop_order' == get_post_type( $sold_record->post_parent ) ) {
                        $sold_count = $sold_count + $sold_record->cnt;
                    }
                }
            }

            return $sold_count;
        }

        /**
         * Get the total number of available tickets per event basis.
         *
         * @param $event_id
         * @return float|int|string|null
         */
        function wc_get_event_tickets_count_left( $event_id ) {

            $event_obj = get_post_meta( $event_id );
            $limit_level = isset( $event_obj[ 'limit_level' ] ) ? $event_obj[ 'limit_level' ][ 0 ] : 0;

            if ( $limit_level ) {
                $max_limit_value = isset( $event_obj[ 'limit_level_value' ] ) ? $event_obj[ 'limit_level_value' ][ 0 ] : 0;
                $event_ticket_sold_count = self::wc_get_event_tickets_count_sold( $event_id );
                return ( $max_limit_value > $event_ticket_sold_count ) ? abs( $max_limit_value - $event_ticket_sold_count ) : 0;

            } else {

                $event = new TC_Event( $event_id );
                $ticket_types = $event->get_event_ticket_types();

                $event_ticket_left_count = 0;

                foreach ( $ticket_types as $ticket_type_id ) {

                    if ( in_array( get_post_type( $ticket_type_id ), [ 'product', 'product_variation' ] )
                        && ! self::ticket_has_child( $ticket_type_id ) ) {

                        $ticket_type = wc_get_product( $ticket_type_id );

                        if ( ! $ticket_type->managing_stock() && 'instock' == $ticket_type->get_stock_status() ) {
                            $event_ticket_left_count = '∞';
                            break;

                        } elseif ( ! $ticket_type->managing_stock() && 'outofstock' == $ticket_type->get_stock_status() ) {
                            $event_ticket_left_count = $event_ticket_left_count + 0;

                        } else {
                            $event_ticket_left_count = $event_ticket_left_count + $ticket_type->get_stock_quantity();
                        }
                    }
                }

                return $event_ticket_left_count;
            }
        }

        /**
         * Identify a ticket if used for variation or simple
         *
         * @param $ticket_type_id
         * @return bool
         */
        function ticket_has_child( $ticket_type_id ) {

            // Init array with variations
            $tickets = get_posts( [
                'post_parent' => $ticket_type_id,
                'post_type' => 'product_variation',
                'post_status' => 'publish',
                'posts_per_page' => -1,
                'fields' => 'ids'
            ]);

            return $tickets ? true : false;
        }

        function tc_custom_event_stats( $atts ) {

            ob_start();

            if ( self::is_bridge_for_woocommerce_active() ) {
                // Render Event Stats with Bridge for Woocommerce
                self::woo_event_stats( $atts );

            } else {
                // Render Event Stats for Tickera Standalone
                self::tc_event_stats( $atts );
            }

            return ob_get_clean();
        }

        /**
         * Check for Bridge for Woocommerce and Woocommerce dependencies
         *
         * @return bool
         */
        function is_bridge_for_woocommerce_active() {
            return ( ( true == apply_filters( 'tc_bridge_for_woocommerce_is_active', false ) || is_plugin_active( 'bridge-for-woocommerce/bridge-for-woocommerce.php' ) ) && is_plugin_active( 'woocommerce/woocommerce.php' ) ) ? true : false;
        }
    }

    $TC_Event_Stats_Shortcode = new TC_Event_Stats_Shortcode();
}
